# Test multiprocess python server
import threading
import logging
import socket
import numpy
import select
import sys, os
sys.path.append(os.path.join(os.path.dirname(__file__), ".."));
import src
import RPi.GPIO as GPIO
from src.managers.robot_manager import Robot
# set the maximum number of attemps to access the server
maxAttempts = 5

# sets the length of the messages sent over the socket
# NOTE: this must match the size on the client end!
MSGLEN = 100

logging.basicConfig(level=logging.DEBUG,
    format='[DEBUG] [%(threadName)-10s] %(message)s',
    )

threads = []

# string to byte array and send
def mysend(sock, message):
    msg = numpy.chararray([100])
    msg.fill('\0')
    i = 0
    for c in message:
        try:
            msg[i] = c
            i = i + 1
        except: pass
    sock.send(msg)

# recv the byte array
def myrecv(sock):
    msg = sock.recv(MSGLEN)
    return msg

# convert strings to arrays and compare
def arrayComp(a, b):
    return str(toCharArray(a).flatten()) == str(toCharArray(b).flatten())

# compare strings to specified character
def arrayCompN(a, b, n):
    result = True
    a = toCharArray(a).flatten()
    b = toCharArray(b).flatten()
    for i in range(n):
        if(a[i] != b[i]):
            result = False
    return result

# converts a string to an array
def toCharArray(aString):
    msg = numpy.chararray([100])
    msg.fill('\0')
    i = 0
    for c in aString:
        try:
            msg[i] = c
            i = i + 1
        except: pass
    return msg

# interact with robot object
def respond(msg, sock, robot):
    sent = 0
    # respond to client queries
    """
    if (arrayCompN(msg, "set:lead screw:retracted", 24)):
        robot.stateLeadScrew = "retracted"
        mysend(sock, "set")
        sent = 1
    if (arrayCompN(msg, "set:lead screw:extended", 23)):
        robot.stateLeadScrew = "extended"
        mysend(sock, "set")
        sent = 1
        """
    if (arrayCompN(msg, "set:top gripper:closed", 22)):
        robot.setGripper("TOP", "closed")
        mysend(sock, "set")
        sent = 1
    if (arrayCompN(msg, "set:bot gripper:closed", 22)):
        robot.setGripper("BOT", "closed")
        mysend(sock, "set")
        sent = 1
    
    if (arrayCompN(msg, "set:top gripper:open", 20)):
        robot.setGripper("TOP", "open")
        mysend(sock, "set")
        sent = 1
    if (arrayCompN(msg, "set:bot gripper:open", 20)):
        robot.setGripper("BOT", "open")
        mysend(sock, "set")
        sent = 1
    
    if (arrayCompN(msg, "state:bot gripper", 17)):
        mysend(sock, robot.stateGripper("BOT"))
        sent = 1
    if (arrayCompN(msg, "state:top gripper", 17)):
        mysend(sock, robot.stateGripper("TOP"))
        sent = 1
    """
    if (arrayCompN(msg, "state:lead screw", 16)):
        mysend(sock, robot.stateLeadScrew)
        sent = 1
    """
    if (arrayCompN(msg, "pot:bot gripper", 15)):
        mysend(sock, str(robot.potGripper("BOT")))
        sent = 1
    if (arrayCompN(msg, "pot:top gripper", 15)):
        mysend(sock, str(robot.potGripper("TOP")))
        sent = 1
        
    if (arrayCompN(msg, "STOP", 4)):
        #Stop fake robot
        mysend(sock, "STOPPING")
        sent = 1
    if (arrayCompN(msg, "ping", 4)):
        mysend(sock, "ping")
        sent = 1
    
    
    # does not understand query
    if (not sent):
        mysend(sock, "What?")
    

# The client thread that the controller connects to
def client_thread(c_sock):
    # this thread simulates the robot for the user client
    # to test the socket communication and data passing
    
    #Fake robot data
    robot = Robot()
    
    # authentication/acknowledge the server
    logging.debug("Starting")
    msg = "Hey"
    mysend(c_sock, msg)
    msg = "Listen!"
    mysend(c_sock, msg)
    msg = ""
    i = 0
    
    read_list = []
    read_list.append(c_sock)
    write_list = []
    write_list.append(c_sock)
    err_list = []
    # listen for client until it disconnects
    while (not arrayCompN(msg,"Bye", 3)) and i < 200:
        msg = ""
        ready_to_read, ready_to_write, in_error = \
               select.select(
                  read_list,
                  write_list,
                  err_list,
                  5)
        read = 0
        for aSocket in ready_to_read:
            msg = myrecv(aSocket)
            logging.debug("msg: " + msg)
            i = i + 1
            respond(msg, c_sock, robot);
        
        robot.driveRobot()
        
    logging.debug("Ending")

# main server thread that watches for incoming connections
def server_thread(port=12345):
    #create an INET, STREAMing socket
    logging.debug("Starting")
    logging.debug("Creating Socket")
    serversocket = socket.socket(
        socket.AF_INET, socket.SOCK_STREAM)
    #bind the socket to a public host,
    # and a well-known port
    logging.debug("Binding to Port: " + str(port))
    serversocket.bind(('', port))
    #become a server socket
    logging.debug("Listening")
    serversocket.listen(5)
    n = 0
    while n < maxAttempts:
        #accept connections from outside
        logging.debug("Waiting for connection")
        (clientsocket, address) = serversocket.accept()
        logging.debug(("Connected to " + str(address)))
        n = n + 1 
        #now do something with the clientsocket
        #in this case, we'll pretend this is a threaded server
        t = threading.Thread(target=client_thread, args=(clientsocket,))
        threads.append(t)
        t.start()
    logging.debug("Exiting")

# Main program
def main():
    logging.info("Starting Server")
    t = threading.Thread(target=server_thread, args=(12344,))
    threads.append(t)
    t.start()

main()
