package client.gui;

import java.awt.Color;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SpringLayout;

import client.util.Util;

/**
 * This class is used to layout the user interface for controlling the robot
 * and originates the messages sent to the robot.
 * 
 * @author will
 *
 */
public class ControlPanel extends JPanel {
	
	/**
	 *  Variables
	 */
	private static final long serialVersionUID = 1L;
	private Window parent;
	
	private SpringLayout layout = new SpringLayout();
	private JLabel title = new JLabel("Robot Controls");
	private JButton back = new JButton("Back");
	
	private JButton stop = new JButton("STOP");
	
	private JLabel topGripper = new JLabel("Top Gripper:");
	private JButton topGripperOpen = new JButton("Open");
	private JButton topGripperClose = new JButton("Close");
	
	private JLabel botGripper = new JLabel("Bot Gripper:");
	private JButton botGripperOpen = new JButton("Open");
	private JButton botGripperClose = new JButton("Close");
	
	private JLabel leadScrew = new JLabel("Lead Screw:");
	private JButton leadScrewExtend = new JButton("Extend");
	private JButton leadScrewRetract = new JButton("Retract");
	
	private JLabel gimbal = new JLabel("Gimbal:");
	private JButton gimbalForward = new JButton("^");
	private JButton gimbalBack = new JButton("V");
	private JButton gimbalLeft = new JButton("<");
	private JButton gimbalRight = new JButton(">");
	private JButton gimbalCenter = new JButton("0");
	
	private JLabel topGripperPot = new JLabel("Top Gripper Pot: ?");
	private JLabel botGripperPot = new JLabel("Bot Gripper Pot: ?");
	

	/**
	 * Create the panel.
	 */
	public ControlPanel(Window parent) {
		this.parent = parent;
		Init();
	}
	
	/**
	 * Interface Layout - uses a spring layout to place interface elements
	 * relative to one another
	 * 
	 *  *in the future this should be broken down into panels with related
	 *   elements
	 */
	private void Init(){
		this.setLayout(layout);
		this.setBackground(Color.WHITE);
		
		// title layout
		this.add(title);
		layout.putConstraint(SpringLayout.WEST, title, 10, SpringLayout.WEST, this);
		layout.putConstraint(SpringLayout.NORTH, title, 10, SpringLayout.NORTH, this);
		
		// next button layout
		this.add(back);
		back.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				parent.showNetwork();
			}
		});
		
		layout.putConstraint(SpringLayout.EAST, back, -10, SpringLayout.EAST, this);
		layout.putConstraint(SpringLayout.SOUTH, back, -10, SpringLayout.SOUTH, this);
		
		// stop button layout
		this.add(stop);
		stop.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				stop();
			}
		});
		layout.putConstraint(SpringLayout.WEST, stop, 10, SpringLayout.WEST, this);
		layout.putConstraint(SpringLayout.SOUTH, stop, -10, SpringLayout.SOUTH, this);
		
		// top gripper
		this.add(topGripper);
		layout.putConstraint(SpringLayout.WEST, topGripper, 0, SpringLayout.WEST, title);
		layout.putConstraint(SpringLayout.NORTH, topGripper, 10, SpringLayout.SOUTH, title);
		
		this.add(topGripperOpen);
		topGripperOpen.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				topGripperOpen();
			}
		});
		topGripperOpen.setEnabled(false);
		layout.putConstraint(SpringLayout.WEST, topGripperOpen, 0, SpringLayout.WEST, topGripper);
		layout.putConstraint(SpringLayout.NORTH, topGripperOpen, 10, SpringLayout.SOUTH, topGripper);
		
		this.add(topGripperClose);
		topGripperClose.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				topGripperClose();
			}
		});
		topGripperClose.setEnabled(false);
		layout.putConstraint(SpringLayout.WEST, topGripperClose, 10, SpringLayout.EAST, topGripperOpen);
		layout.putConstraint(SpringLayout.NORTH, topGripperClose, 10, SpringLayout.SOUTH, topGripper);
		
		
		// bottom gripper
		this.add(botGripper);
		layout.putConstraint(SpringLayout.WEST, botGripper, 0, SpringLayout.WEST, topGripper);
		layout.putConstraint(SpringLayout.NORTH, botGripper, 10, SpringLayout.SOUTH, topGripperOpen);
		
		this.add(botGripperOpen);
		botGripperOpen.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				botGripperOpen();
			}
		});
		botGripperOpen.setEnabled(false);
		layout.putConstraint(SpringLayout.WEST, botGripperOpen, 0, SpringLayout.WEST, botGripper);
		layout.putConstraint(SpringLayout.NORTH, botGripperOpen, 10, SpringLayout.SOUTH, botGripper);
		
		this.add(botGripperClose);
		botGripperClose.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				botGripperClose();
			}
		});
		botGripperClose.setEnabled(false);
		layout.putConstraint(SpringLayout.WEST, botGripperClose, 10, SpringLayout.EAST, botGripperOpen);
		layout.putConstraint(SpringLayout.NORTH, botGripperClose, 10, SpringLayout.SOUTH, botGripper);
		
		// leadScrew
		this.add(leadScrew);
		layout.putConstraint(SpringLayout.WEST, leadScrew, 0, SpringLayout.WEST, botGripper);
		layout.putConstraint(SpringLayout.NORTH, leadScrew, 10, SpringLayout.SOUTH, botGripperOpen);
		
		this.add(leadScrewExtend);
		leadScrewExtend.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				extendScrew();
			}
		});
		leadScrewExtend.setEnabled(false);
		layout.putConstraint(SpringLayout.WEST, leadScrewExtend, 0, SpringLayout.WEST, leadScrew);
		layout.putConstraint(SpringLayout.NORTH, leadScrewExtend, 10, SpringLayout.SOUTH, leadScrew);
		
		this.add(leadScrewRetract);
		leadScrewRetract.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				retractScrew();
			}
		});
		leadScrewRetract.setEnabled(false);
		layout.putConstraint(SpringLayout.WEST, leadScrewRetract, 10, SpringLayout.EAST, leadScrewExtend);
		layout.putConstraint(SpringLayout.NORTH, leadScrewRetract, 10, SpringLayout.SOUTH, leadScrew);
		
		// gimbal controls
		this.add(gimbal);
		layout.putConstraint(SpringLayout.WEST, gimbal, 0, SpringLayout.WEST, leadScrewExtend);
		layout.putConstraint(SpringLayout.NORTH, gimbal, 10, SpringLayout.SOUTH, leadScrewExtend);
		
		this.add(gimbalForward);
		this.add(gimbalBack);
		this.add(gimbalLeft);
		this.add(gimbalRight);
		this.add(gimbalCenter);
		
		layout.putConstraint(SpringLayout.NORTH, gimbalForward, 10, SpringLayout.SOUTH, gimbal);
		layout.putConstraint(SpringLayout.NORTH, gimbalLeft, 10, SpringLayout.SOUTH, gimbalForward);
		layout.putConstraint(SpringLayout.WEST, gimbalLeft, 0, SpringLayout.WEST, gimbal);
		layout.putConstraint(SpringLayout.WEST, gimbalForward, 10, SpringLayout.EAST, gimbalLeft);
		
		layout.putConstraint(SpringLayout.WEST, gimbalCenter, 0, SpringLayout.WEST, gimbalForward);
		layout.putConstraint(SpringLayout.NORTH, gimbalCenter, 0, SpringLayout.NORTH, gimbalLeft);
		layout.putConstraint(SpringLayout.EAST, gimbalCenter, 0, SpringLayout.EAST, gimbalForward);
		layout.putConstraint(SpringLayout.SOUTH, gimbalCenter, 0, SpringLayout.SOUTH, gimbalLeft);
		
		layout.putConstraint(SpringLayout.WEST, gimbalRight, 10, SpringLayout.EAST, gimbalCenter);
		layout.putConstraint(SpringLayout.NORTH, gimbalRight, 0, SpringLayout.NORTH, gimbalCenter);
		layout.putConstraint(SpringLayout.SOUTH, gimbalRight, 0, SpringLayout.SOUTH, gimbalCenter);
		
		
		layout.putConstraint(SpringLayout.WEST, gimbalBack, 0, SpringLayout.WEST, gimbalCenter);
		layout.putConstraint(SpringLayout.EAST, gimbalBack, 0, SpringLayout.EAST, gimbalCenter);
		layout.putConstraint(SpringLayout.NORTH, gimbalBack, 10, SpringLayout.SOUTH, gimbalCenter);
		
		// sensor data
		this.add(topGripperPot);
		layout.putConstraint(SpringLayout.WEST, topGripperPot, 120, SpringLayout.EAST, topGripper);
		layout.putConstraint(SpringLayout.NORTH, topGripperPot, 0, SpringLayout.NORTH, topGripper);
		
		this.add(botGripperPot);
		layout.putConstraint(SpringLayout.WEST, botGripperPot, 0, SpringLayout.WEST, topGripperPot);
		layout.putConstraint(SpringLayout.NORTH, botGripperPot, 5, SpringLayout.SOUTH, topGripperPot);
		
	}
	
	/**
	 * Tells the robot to completely stop a.k.a. emergency stop
	 */
	private void stop(){
		request("STOP");
	}
	
	/**
	 * A local wrapper for calling the request function in the Window class
	 * @param msg - a string to send to the robot and should not exceed the
	 * 				message length for the socket
	 * @return the string that has been sent back by the robot
	 */
	private String request(String msg){
		String ret = parent.request(msg);
		System.out.println("Got: " + ret);
		return ret;
	}
	
	/**
	 * Queries the robot to find the state of the grippers and lead screw,
	 * and pot values for the grippers and gimbal.
	 * 
	 * Using the state data it blocks actions that would not cause a change
	 * to the robots state.(i.e. the user can't open a gripper that is already
	 * opened.)
	 */
	public void refresh(){
		// TODO 
		//state data
		String ret = request("state:top gripper");
		if (Util.strNComp(ret, "closed", 6)){
			System.out.println("Enable open");
			topGripperOpen.setEnabled(true);
			topGripperClose.setEnabled(false);
		}
		if (Util.strNComp(ret, "open", 4)){
			System.out.println("Enable closed");
			topGripperOpen.setEnabled(false);
			topGripperClose.setEnabled(true);
		}
		
		ret = request("state:bot gripper");
		if (Util.strNComp(ret, "closed", 6)){
			System.out.println("Enable open");
			botGripperOpen.setEnabled(true);
			botGripperClose.setEnabled(false);
		}
		if (Util.strNComp(ret, "open", 4)){
			System.out.println("Enable closed");
			botGripperOpen.setEnabled(false);
			botGripperClose.setEnabled(true);
		}
		
		ret = request("state:lead screw");
		if (Util.strNComp(ret, "retracted", 6)){
			System.out.println("Enable extend");
			leadScrewExtend.setEnabled(true);
			leadScrewRetract.setEnabled(false);
		}
		if (Util.strNComp(ret, "extended", 4)){
			System.out.println("Enable retract");
			leadScrewExtend.setEnabled(false);
			leadScrewRetract.setEnabled(true);
		}
		
		// sensor data
		ret = parent.request("pot:top gripper");
		System.out.println("Got: " + ret);
		topGripperPot.setText("Top Gripper: " + ret);
		
		ret = parent.request("pot:bot gripper");
		System.out.println("Got: " + ret);
		botGripperPot.setText("Bot Gripper: " + ret);
	}
	
	
	// the following commands are the standard actions that can
	// be used with the robot
	/**
	 * Sends the command to open the top gripper
	 */
	private void topGripperOpen(){
		request("set:top gripper:open");
		topGripperOpen.setEnabled(false);
		refresh();
	}
	
	/**
	 * Sends the command to close the top gripper
	 */
	private void topGripperClose(){
		request("set:top gripper:closed");
		topGripperClose.setEnabled(false);
		refresh();
	}
	
	/**
	 * Sends the command to open the bottom gripper
	 */
	private void botGripperOpen(){
		request("set:bot gripper:open");
		botGripperOpen.setEnabled(false);
		refresh();
	}
	
	/**
	 * Sends the command to close the bottom gripper
	 */
	private void botGripperClose(){
		request("set:bot gripper:closed");
		botGripperClose.setEnabled(false);
		refresh();
	}
	
	/**
	 * Sends the command to fully extend the lead screw.
	 */
	private void extendScrew(){
		request("set:lead screw:extended");
		leadScrewExtend.setEnabled(false);
		refresh();
	}
	
	/**
	 * Sends the command to fully retract the lead screw.
	 */
	private void retractScrew(){
		request("set:lead screw:retracted");
		leadScrewRetract.setEnabled(false);
		refresh();
	}
	
	/**
	 * Sets the pitch value to x. Where a x of 0 is centered, a positive x angles
	 * the gimbal towards the tree, and a negative x angles the gimbal away from
	 * the tree.
	 * 
	 * @param x - the value to set the gimbal to
	 */
	private void setPitch(int x){
		//TODO
	}
	
	/**
	 * Sets the yaw value to y. Where a y of 0 is centered, a negative y angles
	 * the gimbal to the left, and a positive y angles the gimbal to the right.
	 * 
	 * @param y - the value to set the gimbal to
	 */
	private void setYaw(int y){
		//TODO
	}
	
	/**
	 * Calling this zeros the gimbal by setting the pitch and yaw to zero
	 */
	private void zeroGimbal(){
		setPitch(0);
		setYaw(0);
	}
	
	
	
	
	
}
