package client.gui;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.SpringLayout;


/**
 * This class is the panel for managing the connection to the robot. It lets 
 * the user specify the robots ip and the port at which the to connect the 
 * controller socket to.
 * 
 * @author will
 *
 */
public class NetworkPanel extends JPanel {

	

	/**
	 * Variables
	 */
	private static final long serialVersionUID = 1L;
	private Window parent;
	
	private SpringLayout layout = new SpringLayout();
	private JLabel title = new JLabel("Network Manager");
	private JButton next = new JButton("Next");
	private JLabel robotip = new JLabel("Robot IP:");
	private JTextField ipField = new JTextField();
	private JLabel cPort = new JLabel("Controller Port:");
	private JTextField cPortField = new JTextField();
	private JButton connectController = new JButton("Connect");
	private JButton disConnectController = new JButton("Disconnect");
	private JLabel cStatus = new JLabel("Status: ");
	
	/**
	 * Create the panel.
	 */
	public NetworkPanel(Window parent) {
		this.parent = parent;
		Init();
		
		
	}
	
	/**
	 * General user interface setup
	 */
	private void Init(){
		this.setLayout(layout);
		this.setBackground(Color.WHITE);
		
		// title layout
		this.add(title);
		layout.putConstraint(SpringLayout.WEST, title, 10, SpringLayout.WEST, this);
		layout.putConstraint(SpringLayout.NORTH, title, 10, SpringLayout.NORTH, this);
		
		// next button layout
		this.add(next);
		next.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				parent.showControls();
			}
		});
		next.setToolTipText("Proceed to robot control screen");
		layout.putConstraint(SpringLayout.EAST, next, -10, SpringLayout.EAST, this);
		layout.putConstraint(SpringLayout.SOUTH, next, -10, SpringLayout.SOUTH, this);
		
		// robot ip layout
		this.add(robotip);
		layout.putConstraint(SpringLayout.WEST, robotip, 0, SpringLayout.WEST, title);
		layout.putConstraint(SpringLayout.NORTH, robotip, 30, SpringLayout.NORTH, title);
		this.add(ipField);
		ipField.setPreferredSize(new Dimension(100,15));
		layout.putConstraint(SpringLayout.WEST, ipField, 0, SpringLayout.WEST, robotip);
		layout.putConstraint(SpringLayout.NORTH, ipField, 15, SpringLayout.NORTH, robotip);
		
		// control port
		this.add(cPort);
		layout.putConstraint(SpringLayout.WEST, cPort, 0, SpringLayout.WEST, ipField);
		layout.putConstraint(SpringLayout.NORTH, cPort, 30, SpringLayout.NORTH, ipField);
		
		this.add(cPortField);
		cPortField.setPreferredSize(new Dimension(100,15));
		layout.putConstraint(SpringLayout.WEST, cPortField, 0, SpringLayout.WEST, cPort);
		layout.putConstraint(SpringLayout.NORTH, cPortField, 15, SpringLayout.NORTH, cPort);
		
		// connect button
		this.add(connectController);
		connectController.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				String host = ipField.getText();
				String port = cPortField.getText();
				System.out.println("Attempting to connect to " + host + " via " + port);
				parent.connectController(host, port);
				refresh();
			}
		});
		connectController.setToolTipText("Open connection to specified ip");
		layout.putConstraint(SpringLayout.WEST, connectController, 20, SpringLayout.EAST, cPort);
		layout.putConstraint(SpringLayout.NORTH, connectController, 5, SpringLayout.NORTH, cPort);
		
		// disconnect button
		this.add(disConnectController);
		disConnectController.addActionListener(new ActionListener(){
			@Override
			public void actionPerformed(ActionEvent arg0) {
				// TODO Auto-generated method stub
				System.out.println("Disconnecting controller");
				parent.disconnectController();
				refresh();
			}
		});
		disConnectController.setToolTipText("Close the connection");
		layout.putConstraint(SpringLayout.WEST, disConnectController, 0, SpringLayout.WEST, connectController);
		layout.putConstraint(SpringLayout.NORTH, disConnectController, 10, SpringLayout.SOUTH, connectController);
		

		cStatus = new JLabel("Status: ?");
		this.add(cStatus);
		layout.putConstraint(SpringLayout.WEST, cStatus, 10, SpringLayout.EAST, connectController);
		layout.putConstraint(SpringLayout.NORTH, cStatus, 5, SpringLayout.NORTH, connectController);
		
		/*
		 * sets the default connection to the local loopback interface for ease
		 * of testing server communication
		 */
		ipField.setText("127.0.0.1");
		cPortField.setText("12344");
	}
	
	/**
	 * Query our socket to see if the connection is open
	 */
	public void refresh(){
		if(parent.controllerState()){
			cStatus.setText("Status: CONNECTED");
		}
		else{
			cStatus.setText("Status: NOT CONNECTED");
		}
	}
}
