package client.gui;


import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.EventQueue;

import javax.swing.JFrame;
import javax.swing.SpringLayout;

import client.socket.CommandSocket;

/**
 * This class is the main application window and links the network panel, control
 * panel, and control socket
 * @author will
 *
 */
public class Window extends JFrame{

	/**
	 * Variables
	 */
	private static final long serialVersionUID = 1L;
	private NetworkPanel network = new NetworkPanel(this);
	private ControlPanel control = new ControlPanel(this);
	private CommandSocket controller = null;
	private Container c;
	private SpringLayout spring = new SpringLayout();
	
	/**
	 * Launch the application.
	 */
	public static void main(String[] args) {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					Window window = new Window();
					window.setVisible(true);
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		});
	}

	/**
	 * Create the application.
	 */
	public Window() {
		initialize();
		//while (this.isActive()){
		network.refresh();
		if (controllerState()){
			control.refresh();
		}
			//TODO sleep
		//}
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		this.setBounds(100, 100, 600, 450);
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setTitle("Squirrel Bot 2.1 GUI");
		this.setPreferredSize(new Dimension(600,450));
		c = this.getContentPane();
		c.setBackground(Color.BLACK);
		c.setLayout(spring);
		c.add(network);
		spring.putConstraint(SpringLayout.WEST, network, 5, SpringLayout.WEST, c);
		spring.putConstraint(SpringLayout.NORTH, network, 5, SpringLayout.NORTH, c);
		spring.putConstraint(SpringLayout.EAST, network, -5, SpringLayout.EAST, c);
		spring.putConstraint(SpringLayout.SOUTH, network, -5, SpringLayout.SOUTH, c);
		c.add(control);
		spring.putConstraint(SpringLayout.WEST, control, 5, SpringLayout.WEST, c);
		spring.putConstraint(SpringLayout.NORTH, control, 5, SpringLayout.NORTH, c);
		spring.putConstraint(SpringLayout.EAST, control, -5, SpringLayout.EAST, c);
		spring.putConstraint(SpringLayout.SOUTH, control, -5, SpringLayout.SOUTH, c);
		
		showNetwork();

	}
	
	/**
	 * Switches to the control panel
	 */
	public void showControls(){
		network.refresh();
		if (controllerState()){
			control.refresh();
		}
		
		network.setVisible(false);
		control.setVisible(true);
		
		this.validate();
		this.pack();
	}
	
	/**
	 * Switches to the network panel
	 */
	public void showNetwork(){
		network.refresh();
		if (controllerState()){
			control.refresh();
		}
		
		network.setVisible(true);
		control.setVisible(false);
		
		this.validate();
		this.pack();
	}
	
	/**
	 * Creates a new socket object and opens the connection with the server.
	 * If a socket object exists it will disconnect before creating the new one.
	 * 
	 * @param host - string, target ip
	 * @param port - string, target port
	 */
	public void connectController(String host, String port){
		if(controller != null){
			disconnectController();
		}
		controller = new CommandSocket(host, port);
	    char buffer[] = new char[100];
	    controller.Recv(buffer);
    	System.out.println("Got: " + new String(buffer));
	    controller.Recv(buffer);
    	System.out.println("Got: " + new String(buffer));
	}
	
	/**
	 * Sends the disconnect message and closes the connection
	 */
	public void disconnectController(){
		if(controller != null){
			try{
				controller.Send("Bye");
		    	controller.Close();
			}
			catch(Exception e){
				System.out.println("Error disconnecting");
			}
			finally{
				controller = null;
			}
		}
	}
	
	/**
	 * Wrapper for the request function of the control socket
	 * @param message - string, message to be sent
	 * @return string - the message received from the server
	 */
	public String request(String message){
		return this.controller.Request(message);
	}
	
	/**
	 * Wrapper for checking the state of the control socket 
	 * @return boolean - true if the socket is open
	 */
	public boolean controllerState(){
		boolean state = false;
		if(controller != null){
			state = controller.isConnected();
		}
		return state;
	}
	

}
