package client.socket;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;

/**
 * This class is used to send and receive data for controlling the robot.
 * It provides the following functions
 *  Connect()
 *  Send()
 *  Recv()
 *  Close()
 *  Request()
 *  isConnected()
 * 
 * @author will
 *
 */
public class CommandSocket {
	
	/**
	 * Variables
	 */
	private static int MSGLEN = 100;
	private String host;
	private int port;
	private boolean connected = false;
	
	private Socket kkSocket;
	private PrintWriter out;
	private BufferedReader in;

	
	/**
	 * Initializes the class and tries to open a port
	 * @param host - string, the host ip address
	 * @param port - string, the port at the destination to connect to
	 */
	public CommandSocket(String host, String port){
		// Store destination host and port information
		this.host = host;
		this.port = Integer.parseInt(port);
		
		// Auto connect when class is initialized
		Connect();
		
		// TODO authentication

	}
	
	/**
	 * Opens the connection the the host and port that are stored in this class
	 */
	public void Connect(){
		try{
		    System.out.println("Opening Socket");
		    this.kkSocket = new Socket(this.host, this.port);
		    this.out = new PrintWriter(kkSocket.getOutputStream(), true);
		    this.in = new BufferedReader(
		        new InputStreamReader(kkSocket.getInputStream()));
		    System.out.println("Connected to " + host);
		    this.connected = kkSocket.isConnected();
		} catch (Exception e){
			System.out.println("Failed to connect");
		}
	}
	
	/**
	 * Closes the socket if a socket is open
	 */
	public void Close(){
		this.connected = kkSocket.isConnected();
		if(connected){
			try{
				this.kkSocket.close();
				System.out.println("Closed connection");
			}
			catch(Exception e){
				System.out.println("Failed to close connection");
			}
		}
	}
	
	/**
	 * Send function for this socket. It takes a string and converts it into a 
	 * byte array and truncates at specified message length
	 * 
	 * @param message - a string to be sent over the network
	 * @return integer - returns 0 on successful send
	 */
	public int Send(String message){
		this.connected = kkSocket.isConnected();
		if(connected){
		    char buffer[] = new char[MSGLEN];
		    int c = 0;
		    while (c < MSGLEN) {
		    	buffer[c] = '\0';
		    	c++;
		    }
		    char strArray[] = message.toCharArray();
		    int i = 0;
		    while (i < MSGLEN && i < strArray.length){
		    	buffer[i] = strArray[i];
		    	i++;
		    }
		    try{
		    	out.write(buffer, 0, buffer.length);
		    	out.flush();
		    	System.out.println("Sent: " + new String(buffer));
		    	return 0;
		    } catch (Exception e){
				System.out.println("Failed to send");
		    	return 1;
		    }
		}
		System.out.println("Failed to send");
		return 1;
	}
	
	/**
	 * Receive function for the socket. It gets the bytes sent from the server
	 * and places them into a buffer.
	 * NOTE: only handles a single buffer at a time
	 * 
	 * @param buffer - a char array for storing the incoming data
	 * @return integer - returns 0 on successful recv
	 */
	public int Recv(char buffer[]){
		this.connected = kkSocket.isConnected();
		if (connected){
			try{
				in.read(buffer, 0, buffer.length);
		    	System.out.println("Recieved: " + new String(buffer));
		    	return 0;
			} catch (Exception e){
				System.out.println("Failed to recv");
				return 1;
			}
		}
		System.out.println("Failed to recv");
		return 1;
	}
	
	/**
	 * Send a message that expects to be answered and only waits for the answer
	 * if the send call was successful
	 * Use this instead of send as the sockets on both the client and server
	 * are blocking sockets
	 * 
	 * @param message - string, a query for the server in the form of a string
	 * @return string - the response from the server
	 */
	public String Request(String message){
		char buffer[] = new char[MSGLEN];
		if(Send(message) == 0){
			Recv(buffer);
		}
		return new String(buffer);
	}
	
	/**
	 * Check if the socket is still connected and return true if it is.
	 * 
	 * @return boolean - returns true if the socket is still usable
	 */
	public boolean isConnected(){
		this.connected = kkSocket.isConnected();
		return this.connected;
	}

}
